package com.qizhen.healsphere.repository.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.qizhen.healsphere.web.param.CladeCountResult;
import com.qizhen.healsphere.web.param.Epiflu;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Mapper
public interface EpifluMapper extends BaseMapper<Epiflu> {
    // 获取subtype的列表
    @Select("SELECT DISTINCT subtype FROM epiflu")
    List<String> findDistinctSubtype();

    @Select("SELECT DISTINCT clade FROM epiflu WHERE subtype = #{subtype}")
    List<String> findDistinctCladeBySubtype(@Param("subtype") String subtype);

    @Select("SELECT COUNT(1) FROM epiflu WHERE subtype = #{subtype}")
    Integer selectCountBySubtype(@Param("subtype") String subtype);

    @Select("SELECT MAX(collection_date) FROM epiflu WHERE subtype = #{subtype}")
    Date findMaxCollectionDateBySubtype(@Param("subtype") String subtype);

    @Select("SELECT COUNT(1) FROM epiflu WHERE subtype = #{subtype} AND collection_date BETWEEN #{startDate} AND #{endDate}")
    Integer selectCountBySubtypeAndDateRange(@Param("subtype") String subtype, @Param("startDate") LocalDate startDate, @Param("endDate") LocalDate endDate);

    @Select("SELECT COUNT(1) FROM epiflu WHERE subtype = #{subtype} AND collection_date BETWEEN #{startDate} AND #{endDate} AND location LIKE CONCAT('%', #{continent}, '%')")
    Integer selectCountBySubtypeAndDateRangeWithContinent(@Param("subtype") String subtype, @Param("startDate") LocalDate SeasonStartDate, @Param("endDate") LocalDate SeasonEndDate, @Param("continent") String continent);

//    @Select("<script>" +
//            "SELECT DISTINCT clade " +
//            "FROM epiflu " +
//            "WHERE subtype = #{subtype} " +
//            "AND is_china = #{isChina} " +
//            "AND collection_date BETWEEN #{startDate} AND #{endDate} " +
//            "<if test='constellation != null and constellation.size() > 0'>" +
//            "</script>")
//    List<String> findDistinctCladeBySearchWithConditions(@Param("subtype") String subtype, @Param("isChina") Integer isChina, @Param("startDate") LocalDate startDate, @Param("endDate") LocalDate endDate);

    /**
     * 查询指定洲的clade分布
     * @param continent 洲名称
     * @return clade统计列表
     */
    @Select("SELECT clade, COUNT(*) AS count " +
            "FROM public.epiflu " +
            "WHERE continent_location = #{continent} " +
            "AND subtype = #{subtype} " +
            "GROUP BY clade " +
            "ORDER BY count DESC")
    List<CladeCountResult> selectCladeCountByContinent(@Param("continent") String continent , @Param("subtype") String subType);

    /**
     * 查询指定日期、子类型和星座的记录数
     * @param subtype 子类型
     * @param constellation 星座
     * @param date 日期
     * @return 记录数
     */
    @Select("SELECT COUNT(*) FROM epiflu WHERE subtype = #{subtype} AND clade = #{constellation} AND collection_date = #{date}")
    Integer selectDayCount(@Param("subtype") String subtype, @Param("constellation") String constellation, @Param("date") LocalDate date);

    @Select("SELECT DISTINCT subtype FROM epiflu WHERE subtype = #{is_china}")
    List<String> findDistinctSubtypeChina();

    @Select("SELECT COUNT(*) FROM epiflu WHERE subtype = #{subtype} AND is_china = #{is_china} AND collection_date = #{startDate}")
    Integer selectCount(@Param("subtype") String subtype, @Param("is_china") Integer is_china, @Param("startDate") LocalDate startDate);

}